/*
BREAKS.PRG

Bruce E. Hansen
Department of Economics
Social Science Building
University of Wisconsin
Madison, WI 53706-1393
bhansen@ssc.wisc.edu
http://www.ssc.wisc.edu/~bhansen/

Replicates empirical work from
"The New Econometrics of Structural Change:
Dating Breaks in U.S. Labor Productivity" 
by Bruce E. Hansen

*/

_seas=12;				@ seasonal frequency of data	@
k=1;				@ AR order 		@
_trim=.03;				@ trim % for estimation 	@
_trimsup=.05;			@ trim % for sup test		@
_trimexp=.05;			@ trim % for exp test		@
_bootw=0;			@ # of fixed-regressor bootstrap replications (set to 0 if not desired) @
_conf=.9;				@ Confidence Level 		@

output file = ip.out reset;		@ name of output file		@
_tit = "Labor Productivity - ";	@ string: title		@

indx = 1 ;				@ variable index - 1 to 11	@
load dat1[652,11]=ip.dat;
load dat2[652,11]=workers.dat;
load dat3[652,11]=hours.dat;
dat=ln(dat1)-ln(dat2)-ln(dat3);
dat=packr(dat[.,indx]);
dat=(dat[2:rows(dat)]-dat[1:rows(dat)-1])*_seas*100;
year_1=1947+1/12;
year_2=year_1-1/12+rows(dat)/_seas;

sample_1=year_1;			@ first year to use in sample : should be "year_1" or larger @
sample_2=year_2;

tr1=_seas*(sample_1-year_1);
tr2=_seas*(year_2-sample_2);
dy=dat[1+tr1:rows(dat)-tr2];
n=rows(dy);
year1=seqa(sample_1+1/_seas,1/_seas,n); 	@ "date" vector, should correspond to data in "dy" @

let _tit2 =
"Durables"
"SIC24"
"SIC25"
"SIC32"
"SIC33"
"SIC34"
"SIC35"
"SIC36"
"SIC37"
"SIC38"
"SIC39";

/********************************************/

_title = "" $+ _tit $+ _tit2[indx];

graphset;
_pdate="";
_pcolor=15;
_plwidth=2;
fonts("complex");
title(_title);
ylabel("Percentage Change - Annual Rate");
xy(year1,dy);

#include pv_sup.prc;
#include pv_exp.prc;
#include st_chang.prc;

@ Constant Parameter Model @
yy=dy[k+1:n];
year1=year1[k+1:n];
t=n-k;
tr=seqa(1,1,t);
ot=ones(t,1);
xar=x_ar(dy,k);
x=xar~ot;
rho=yy/x;
e0=yy-x*rho;
ee0=e0'e0;
sig=ee0/(t-k-1);
xx=invpd(moment(x,0));
v=xx*moment(x.*e0,0)*xx;
se=sqrt(diag(v));

_names=0 $+ "DY(-" $+ ftocv(seqa(1,1,k),1,0) $+ ")";
_names=_names|"C";
let fmt1[1,3]="*.*lf" 12 8;
let fmt2[1,3]="*.*lf" 12 6;
let fmt3[1,3]="*.*lf" 16 8;
$_title;
"Sample for years " sample_1~sample_2;
"Number of AR Lags: " k;"";
"Constant Parameter AR Model";
pr=printfm("Variable"~"Estimate"~"St Error",0~0~0,fmt1);"";
pr=printfm(_names~rho~se,0~1~1,fmt2);
pr=printfm("sigma"~sqrt(sig),0~1,fmt2);"";"";
"************************************";
"";
output off;

/***************************************/
@ Structural Change Model @

{kest,ss}=splitest(yy,x,0,_trim);
d=(tr.<=kest);
y1=selif(yy,d);
x1=selif(x,d);
t1=rows(y1);
y2=selif(yy,1-d);
x2=selif(x,1-d);
t2=rows(y2);

@ Regime 1 Estimates @
rho1=y1/x1;
e1=y1-x1*rho1;
ee1=e1'e1;
sig1=ee1/(t1-k-1);
q1=moment(x1,0);
xx=invpd(q1);
q1=q1/t1;
v1=xx*moment(x1.*e1,0)*xx;
se1=sqrt(diag(v1));
sumrho=1-sumc(rho1[1:k]);
mu1=rho1[k+1]/sumrho;
h1=((ones(k,1)*mu1)|1)/sumrho;
vmu1=h1'v1*h1;

@ Regime 2 Estimates @
rho2=y2/x2;
e2=y2-x2*rho2;
ee2=e2'e2;
sig2=ee2/(t2-k-1);
q2=moment(x2,0);
xx=invpd(q2);
q2=q2/t2;
v2=xx*moment(x2.*e2,0)*xx;
se2=sqrt(diag(v2));
mu2=rho2[k+1]/sumrho;
h2=((ones(k,1)*mu2)|1)/sumrho;
vmu2=h2'v2*h2;

tm=(mu2-mu1)/sqrt(vmu1+vmu2);
e=e1|e2;

@ Confidence Interval for Breakdate @
delta=rho2-rho1;
xsi=(delta'q2*delta)/(delta'q1*delta);
phi=xsi*sig2/sig1;
{c1,c2}=bai_qnt(xsi,phi,1-_conf);
ll=(delta'q1*delta)/sig1;
k1=kest-1-ceil(c2/ll);
k2=kest+1-floor(c1/ll);
k1=maxc(k1|1);
k2=minc(k2|t);

@ Graph SSE @
graphset;
_pdate="";
_pcolor=15;
_plwidth=2;
fonts("complex");
ylabel("Residual Variance");
xlabel("Breakdate");
xtics(1950,2000,5,5);
tit = "" $+ _title $+ "\LEstimation of Breakdate in Regression";
tit = "" $+ "Least Squares Breakdate Estimation:\LResidual Variance as a Function of Breakdate";
title(tit);
ss=ss/t;
xy(year1,ss);

output on;
"Complete Structural Change AR Model";
"";
"Estimated Breakdate " year1[kest];
"Confidence Interval " year1[k1]~year1[k2];
"Confidence % " _conf ;
"Trimming Percentage  " _trim;
"t test for equality of means " tm;
"";
"Regime 1";
"Mean     " mu1~sqrt(vmu1);
pr=printfm("Variable"~"Estimate"~"St Error",0~0~0,fmt1);"";
pr=printfm(_names~rho1~se1,0~1~1,fmt2);
pr=printfm("sigma"~sqrt(sig1),0~1,fmt2);"";"";

"Regime 2";
"Mean     " mu2~sqrt(vmu2);
pr=printfm("Variable"~"Estimate"~"St Error",0~0~0,fmt1);"";
pr=printfm(_names~rho2~se2,0~1~1,fmt2);
pr=printfm("sigma"~sqrt(sig2),0~1,fmt2);"";"";
"************************************";
"";

/***************************************/
@ Testing for Structural Change @
@ Compute W Statistics @
{f,supf,expf,kest}=_supw(yy,x,_trimsup,_trimexp);

@ Graph F Statistics@
cv=((zeros(k+1,1)+qa_crit(1,_trimsup))|qa_crit(k+1,_trimsup))~((zeros(k+1,1)+chi_crit(1))|chi_crit(k+1));
_names2=_names|"Joint";
graphset;
_pdate="";
_pcolor=15;
_plwidth=3;
let _pltype = 6 4 3;
fonts("complex simgrma");
xlabel("Breakdate");
ylabel("Chow Test");
let _plegctl = 1;
_plegstr="Chow Test Sequence\000Andrews Critical Value\000\202h\201[2] Critical Value";
j=1; do while j<=k;
  tit = "" $+ "Testing for Structural Change\L" $+ _title $+ "\L " $+ _names2[j];
  title(tit);
  z0=ones(t,1)*cv[j,.];
  xy(year1,f[.,j]~z0);
j=j+1;endo;
tit = "" $+ "Testing for Structural Change\L" $+ _title $+ "\LIntercept";
title(tit);
z0=ones(t,1)*cv[k+1,.];
xy(year1,f[.,k+1]~z0);

ytics(0,24,4,4);
xtics(1950,2000,5,5);
let _plegctl[1,4]=1 5 1950 19;
tit = "" $+ "Testing for Structural Change of Unknown Timing:\LChow Test Sequence as a Function of Breakdate";
title(tit);
z0=ones(t,1)*cv[k+2,.];
xy(year1,f[.,k+2]~z0);

@ Andrews' P-Values @
pvs1=zeros(k+2,1);
pve1=zeros(k+2,1);
j=1; do while j<=k+1;
  pvs1[j]=pv_sup(supf[j],1,_trimsup);
  pve1[j]=pv_exp(expf[j],1,_trimexp);
j=j+1;endo;
pvs1[k+2]=pv_sup(supf[k+2],k+1,_trimsup);
pve1[k+2]=pv_exp(expf[k+2],k+1,_trimexp);

"Test Breakdates";"";
pr=printfm("Variable"~"BreakEst",zeros(1,2),fmt3);"";
pr=printfm(_names2~year1[kest],0~1,fmt3);"";"";
"Trimming Percentage for SupW " _trimsup;
"Trimming Percentage for ExpW " _trimexp;
"";"";


statfs=_names2~supf~pvs1;
statfe=_names2~expf~pve1;
vnames="Variable"~"SupW"~"AsyP";
vnamee="Variable"~"ExpW"~"AsyP";

if _bootw >0;
  "Number of Fixed Regressor Bootstrap Replications " _bootw;
  @   Fixed Regressor Bootstrap @
  boots=zeros(_bootw,k+2);
  boote=zeros(_bootw,k+2);
  b=1; do while b<=_bootw;
    yb=e.*rndn(t,1);
    {fb,supfb,expfb,kb}=_supw(yb,x,_trimsup,_trimexp);
    boots[b,.]=supfb';
    boote[b,.]=expfb';
  b=b+1;endo;
  pvs2=meanc(boots.>(supf'));
  pve2=meanc(boote.>(expf'));
  statfs=statfs~pvs2;
  statfe=statfe~pve2;
  vnames=vnames~"FixRegP";
  vnamee=vnamee~"FixRegP";
endif;

@ Print Test Results @
output on;
"SupW Tests on Regression Parameters";"";
pr=printfm(vnames,zeros(1,cols(vnames)),fmt1);"";
pr=printfm(statfs,0~ones(1,cols(statfs)-1),fmt2);"";"";

"ExpW Tests on Regression Parameters";"";
pr=printfm(vnamee,zeros(1,cols(vnamee)),fmt1);"";
pr=printfm(statfe,0~ones(1,cols(statfe)-1),fmt2);
output off;

/**********************************
Variance Equation  - LS Residuals
**********************************/

e2=e0.*e0;
{kest,ss}=splitest(e2,ot,0,_trim);
d=(tr.<=kest);
xx=d~(1-d);
sigs=e2/xx;
u=e2-xx*sigs;
u1=u[1:kest];u2=u[kest+1:t];
t1=kest; t2=t-kest;
sig1=(u1'u1)/(t1-k-1);
sig2=(u2'u2)/(t2-k-1);
{c1,c2}=bai_qnt(1,1,1-_conf);
ll=sig1/((sigs[1]-sigs[2])^2);
k1=kest-1-ceil(c2*ll);
k2=kest+1-floor(c1*ll);
k1=maxc(k1|1);
k2=minc(k2|t);

output on;
"********************************************";
"";"";
"Break in Error Variance ";
"Estimated Breakdate " year1[kest];
"Confidence Interval " year1[k1]~year1[k2];
"Confidence % " _conf ;
"Regime 1 SD " sqrt(sigs[1]);
"Regime 2 SD " sqrt(sigs[2]);
"";"";
"********************************************";
"";"";
output off;

@ Graph SSE @
graphset;
_pdate="";
_pcolor=15;
_plwidth=2;
fonts("complex");
ylabel("Residual Variance");
xlabel("Breakdate");
tit = "" $+ _title $+ "\LEstimation of Breakdate in Variance";
title(tit);
ss=ss/t;
xy(year1,ss);

@ Test Constancy of Variance @
{f,supf,expf,kest}=_supw(e2,ot,_trimsup,_trimexp);
supf=supf[1];expf=expf[1];kest=kest[1];
tests=supf|expf;
pvvar=pv_sup(supf,1,_trimsup)|pv_exp(expf,1,_trimexp);
cvvar=qa_crit(1,_trimsup);

stat1=(""~"Stat"~"AsyP");
stat2=(("SupF"|"ExpF")~tests~pvvar);
let fmt1[1,3]="*.*lf" 12 8;
let fmt2[1,3]="*.*lf" 12 6;
let fmt3[1,3]="*.*lf" 16 8;

output on;
"Tests for Constancy of Error Variance ";
"";"";
pr=printfm(stat1,zeros(1,cols(stat1)),fmt1);"";
pr=printfm(stat2,0~ones(1,cols(stat2)-1),fmt2);"";
"Test Breakdate " year1[kest];
"";"";
"********************************************";
"";"";
output off;

@ Graph F Statistics@
graphset;
_pdate="";
_pcolor=15;
_plwidth=2;
fonts("complex");
_plegctl=1;
_plegstr="Wald Sequence\000Asymptotic Critical Value";
  tit = "" $+ _title $+ "\LWald Sequence for Variance";
  title(tit);
  z0=ones(t,1)*cvvar;
  xy(year1,f[.,1]~z0);

/**********************************************/
